# -*- coding: utf-8 -*-
#
#------------------------------------------------------------------------------
# TrasMiSIG  -  Software libre para consultar y visualizar rutas ptimas en una
# red de transporte masivo compuesta por estaciones y segmentos. Utiliza
# PostgreSQL/PostGIS y PyQGIS.
#
# Copyright (C) 2009  Germn Carrillo
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

from PyQt4.QtSql import QSqlDatabase, QSqlQuery

from config import*

class ConexionPgSQL():
    """ Provee los mtodos para conectarse a PostgreSQL/PostGIS """
    def __init__( self ):

        # Destruir el objeto si ya existe. Se usa el mtodo esttico.
        if QSqlDatabase.contains( "bdTrasmisig" ):
            QSqlDatabase.removeDatabase( "bdTrasmisig" )

        # Paramentros conexion
        self.db = QSqlDatabase.addDatabase( "QPSQL", "bdTrasmisig" )
        self.db.setHostName( SERVIDOR )
        self.db.setPort( int( PUERTO ) )
        self.db.setDatabaseName( BD )
        self.db.setUserName( USUARIO )
        self.db.setPassword( CONTRASENA )

        self.query = QSqlQuery( self.db )
        self.conectado = self.conectar()

    def conectar( self ):
        """ Abre la conexin a la base de datos"""
        conectado = self.db.open()
        return conectado

    def consultarNombresEstaciones( self ):
        """ Retorna los nombres de las estaciones para inicializar combos """
        if self.query.exec_( "SELECT estacion.nombreEstacion AS Nombre FROM " \
            + "estacion;" ):

            aTabla = []
            while self.query.next():
                nombre = self.query.value( 0 ).toString()
                aTabla.append( nombre )
            return aTabla
        else:
            return None

    def consultarServicios( self, sOrigen, sDestino, sDia, sH, sM ):
        """ Retorna id's de servicios que pasan por estaciones origen y destino """
        sHora = sH + ":" + sM

        if self.query.exec_( "SELECT * FROM encontrarServicios('" \
            + sOrigen  + "' , '" + sDestino + "', '" + sDia + "', '" \
            + sHora + "');" ):

            record = self.query.record()
            posId = record.indexOf( "idServicio" )
            posNombre = record.indexOf( "Nombre" )
            posTipo = record.indexOf( "tipoServicio" )

            aTabla = []
            while self.query.next():
                iId = self.query.value( posId ).toString()
                nombre = self.query.value( posNombre ).toString()
                tipo = self.query.value( posTipo ).toString()
                aTabla.append( [ iId, nombre, tipo ] )
            return aTabla
        else:
            return None

    def calcularTiempo( self, idServicio, sOrigen, sDestino ):
        """ Devuelve el tiempo que tarda el viaje en minutos """
        if self.query.exec_( "SELECT calcularTiempo(" + idServicio + ",'" \
            + sOrigen + "','" + sDestino + "');" ):

            self.query.next()
            tiempo = self.query.value( 0 ).toString()
            return int( tiempo ) # El entero sirve para ordenar
        else:
            return None

    def calcularNoParadas( self, idServicio, sOrigen, sDestino ):
        """ Devuelve el nmero de paradas intermedias """
        if self.query.exec_( "SELECT calcularNoParadas(" + idServicio + ",'" \
            + sOrigen + "','" + sDestino + "');" ):

            self.query.next()
            paradas = self.query.value( 0 ).toString()
            return paradas
        else:
            return None

    def crearVistaEstaciones( self, sOrigen, sDestino ):
        """ Crea la vista que contiene las estaciones de origen y destino """
        if self.query.exec_( "SELECT CrearVistaEstaciones('" + sOrigen + "','" \
            + sDestino + "');" ):
            return True
        else:
            return False

    def crearVistaParadas( self, idServicio, sOrigen, sDestino ):
        """ Crea la vista que contiene las paradas intermedias """
        if self.query.exec_( "SELECT CrearVistaParadas(" + idServicio + ",'" \
            + sOrigen + "','" + sDestino + "');" ):
            return True
        else:
            return False

    def crearVistaRuta( self, idServicio, sOrigen, sDestino ):
        """ Crea la vista que contiene los segmentos que forman la ruta """
        if self.query.exec_( "SELECT CrearVistaRuta(" + idServicio + ",'" \
            + sOrigen + "','" + sDestino + "');" ):
            return True
        else:
            return False